import 'package:easy_pass_common/dto/jwt_token.dart';
import 'package:easy_pass_common/dto/user.dart';
import 'package:easy_pass_common/service/authentication_service.dart';
import 'package:easy_pass_common/service/service_base/status.dart';
import 'package:easy_pass_common/service/user_service.dart';
import 'package:flutter/material.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:get_it/get_it.dart';

class Session with ChangeNotifier {
  Status status = Status.Idle;
  final AuthenticationService authenticationService = GetIt.instance<AuthenticationService>();
  final UserService userService = GetIt.instance<UserService>();
  User loggedInUser;
  String userEmail;
  String userPassword;

  Future login() async {
    status = Status.Busy;
    notifyListeners();
    var response = await authenticationService.login(userEmail, userPassword);
    loggedInUser = response.user;
    await _persistSessionToStorage(response.jwt);

    status = Status.Idle;
    notifyListeners();  
  }

  Future logout() async {
    status = Status.Busy;
    notifyListeners();
    loggedInUser = null;
    await _removeSessionFromStorage();

    status = Status.Idle;
    notifyListeners();  
  }

  setUserEmail(String userEmail) {
    this.userEmail = userEmail;
  }

  Future loadFromLocalStorage() async {
    try 
    {
      final storage = FlutterSecureStorage();
      final jwtString = await storage.read(key: 'jwt');
      if (jwtString == null) {
        return false;
      }
      final jwtToken = JwtToken.fromString(jwtString);
      if (jwtToken == null) {
        return false;
      }
      loggedInUser = await userService.getUser(jwtToken.userId);
      if (loggedInUser == null) {
        return false;
      }
      return true;
    }
    catch(exception) {
      print (exception);
      return false;
    }
  }

  setUserPassword(String userPasswrod) {
    this.userPassword = userPasswrod;
  }

  Future _persistSessionToStorage(String jwt) async {
    var storage = FlutterSecureStorage();

    await storage.write(key: 'jwt', value: jwt);       
  }

  Future _removeSessionFromStorage() async {
    var storage = FlutterSecureStorage();

    await storage.delete(key: 'jwt');
  }
}