
import 'package:easy_pass_common/exceptions/unauthorized_exception.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http/http.dart';
import 'package:http/testing.dart';

import '../../fixture_reader.dart';
import '../../helper/service_test_helper.dart';
import 'test_get_request.dart';
import 'test_service.dart';

void main() {
  setUpAll(() async {
    ServiceTestHelper.setup();
  });

  group("Base service - ", () {
    test("index - will make a get request with parameters", () async {

      var mockClient = MockClient((request) async {
        return Response(fixture('test_list_response.json'), 200);
      });            

      TestService testService = TestService.withClient(mockClient);
      var testDtos = await testService.getListObjects();

      expect(testDtos, isNotNull);
      expect(testDtos.length, equals(2));

      expect(testDtos[0].id, equals(1));
      expect(testDtos[0].field1, equals('test-value-1'));

      expect(testDtos[1].id, equals(2));      
      expect(testDtos[1].field1, equals('test-value-2'));
    });

    test("geti - will throw an UnAuthorised exception on a 401 response", () async {

      var mockClient = MockClient((request) async {
        return Response(fixture('test_single_response.json'), 401);
      });            

      TestService testService = TestService.withClient(mockClient);

      expect(
        () async => await testService.geti(TestGetRequest()), 
        throwsA(isInstanceOf<UnAuthorizedException>())
      );
    });    
  });
}